import * as METADATA from '../constants';

export interface Config {
  qualifier?: string;
  scope?: Scope;
}

export enum Scope {
  SINGLETON = 1,
  PROTOTYPE,
}

const injectable = (config?: Config) => {
  return <TFunction extends Function>(target: TFunction): TFunction | void => {

    if (Reflect.hasOwnMetadata(METADATA.PARAM_TYPES, target) === true) {
      throw new Error('Cannot apply @injectable decorator multiple times.');
    }

    let types = Reflect.getMetadata(METADATA.DESIGN_PARAM_TYPES, target);
    Reflect.defineMetadata(METADATA.PARAM_TYPES, types, target);

    const name = target.name;

    return target;
  };
};

export default injectable;
