import Action from '@overflow/commons/redux/Action';
import Probe from '../../api/model/Probe';
import ReadAllByDomainPayload from '../payload/ReadAllByDomainPayload';
import Domain from '@overflow/domain/api/model/Domain';

// Action Type
export type REQUEST = '@overflow/probe/read_all_by_domain/REQUEST';
export type REQUEST_SUCCESS = '@overflow/probe/read_all_by_domain/REQUEST_SUCCESS';
export type REQUEST_FAILURE = '@overflow/probe/read_all_by_domain/REQUEST_FAILURE';

export const REQUEST: REQUEST = '@overflow/probe/read_all_by_domain/REQUEST';
export const REQUEST_SUCCESS: REQUEST_SUCCESS = '@overflow/probe/read_all_by_domain/REQUEST_SUCCESS';
export const REQUEST_FAILURE: REQUEST_FAILURE = '@overflow/probe/read_all_by_domain/REQUEST_FAILURE';

// Action Creater
export type request = (domain: Domain) => Action<ReadAllByDomainPayload>;
export type requestSuccess = (probes: Probe[]) => Action<Probe[]>;
export type requestFailure = (error: Error) => Action;



export const request: request = (domain: Domain): Action<ReadAllByDomainPayload> => {
  return {
    type: REQUEST,
    payload: {
      domain: domain,
    },
  };
};

export const requestSuccess: requestSuccess = (probes: Probe[]): Action<Probe[]> => {
  return {
    type: REQUEST_SUCCESS,
    payload: probes,
  };
};

export const requestFailure: requestFailure = (error: Error): Action => {
  return {
    type: REQUEST_FAILURE,
    error: error,
  };
};
