import * as React from 'react';
import { Table, Button, Modal, Input, Header, Container, InputOnChangeData, ButtonProps } from 'semantic-ui-react';
import { TargetDetails } from './TargetDetails';

import Target from '@overflow/target/api/model/Target';

export interface TargetsProps {

}

export interface TargetsState {
  selected: Target;
  openAddTarget: boolean;
}

export class Targets extends React.Component<TargetsProps, TargetsState> {

  constructor(props: TargetsProps, context: TargetsState) {
    super(props, context);
    // this.state = {
    //   openAddTarget: false,
    // };
  }

  public handleAddTarget(event: React.MouseEvent<HTMLButtonElement>, data: ButtonProps): void {
    this.setState({
      openAddTarget: true,
    });
  }

  public render(): JSX.Element {
    const options = [
      { key: 'm', text: 'Male', value: 'male' },
      { key: 'f', text: 'Female', value: 'female' },
    ];

    return (
      <Container fluid>
        {/*search bar  */}
        <TargetTable />
        <br />
        <Button content='Add' icon='add' labelPosition='left' floated='right' positive onClick={this.handleAddTarget.bind(this)} />
        <br /><br /><br /><br /><br /><br /><br /><br /><br />
        <TargetDetails target={this.state.selected} />

        <Modal
          open={this.state.openAddTarget}
        >
          <Modal.Header>
            Adding a Target
          </Modal.Header>
          <Modal.Content>
            <p>contents</p>
          </Modal.Content>
          <Modal.Actions>
            <Button negative onClick={() => this.setState({ openAddTarget: false })}>Cancel</Button>
            <Button positive labelPosition='right' icon='checkmark' content='Done' />
          </Modal.Actions>
        </Modal>
      </Container>
    );
  }

  public componentWillMount(): void {
    super.componentWillMount();
  }
}

export interface TargetTableProps {

}

export interface TargetTableState {
  list: Array<Target>;
  selected: Target;
}

export class TargetTable extends React.Component<TargetTableProps, TargetTableState> {
  private data: any;

  constructor(props: TargetTableProps, context: TargetTableState) {
    super(props, context);
    this.state = {
      selected: null,
      list: [],
    };
  }
  public handleSelect(selectedTarget: Target): void {
    this.setState({
      selected: selectedTarget,
    });
  }

  public handleSearch(event: React.SyntheticEvent<HTMLInputElement>, data: InputOnChangeData): void {
    let searchWord = data.value.toLowerCase();
    let foundTarget = new Array();
    for (let target of this.data) {
      let typeName = target.infra.metaInfraType.name.toLowerCase();
      if (typeName.indexOf(searchWord) !== -1) {
        foundTarget.push(target);
      }
    }
    this.setState({
      list: foundTarget,
    });
  }

  public render(): JSX.Element {
    return (
      <Container fluid>
        <Header as='h3' dividing>Targets</Header>
        <Input icon='search' placeholder='Search...' onChange={this.handleSearch.bind(this)} />
        <Table celled selectable striped>
          <Table.Header>
            <Table.Row>
              <Table.HeaderCell textAlign={'center'}>No.</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Type</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Name</Table.HeaderCell>
              <Table.HeaderCell textAlign={'center'}>Version</Table.HeaderCell>
            </Table.Row>
          </Table.Header>

          <Table.Body>
            {this.state.list.map((target: Target, index: number) => (
              <Table.Row key={index} onClick={this.handleSelect.bind(this, target)}>
                <Table.Cell textAlign={'center'}>{index + 1}</Table.Cell>
                <Table.Cell textAlign={'center'}>{target.infra.type.name}</Table.Cell>
                <Table.Cell>todo</Table.Cell>
                <Table.Cell>todo</Table.Cell>
              </Table.Row>
            ))}
          </Table.Body>
        </Table>
      </Container>
    );
  }

  public componentWillMount(): void {
    super.componentWillMount();
    this.data = [
      {
        'id': '1',
        'createDate': '',
        'probe': {
          'id': '1',
        },
        'infra': {
          'id': '1',
          'metaInfraType': {
            'id': '1',
            'name': 'OS',
            'createDate': '424252',
          },
          'childId': '1',
          'createDate': '13345235',
        },
      },
      {
        'id': '2',
        'createDate': '',
        'probe': {
          'id': '1',
        },
        'infra': {
          'id': '1',
          'metaInfraType': {
            'id': '1',
            'name': 'Host',
            'createDate': '424252',
          },
          'childId': '1',
          'createDate': '13345235',
        },
      },
    ];
    this.setState({
      list: this.data,
    });
  }
}



